#!/usr/bin/env python

'''Avena is an image processing library and command-line tool.'''


__author__ = 'Mansour Moufid'
__copyright__ = 'Copyright 2014, 2015, Mansour Moufid'
__license__ = 'ISC'
__version__ = '0.5'
__email__ = 'mansourmoufid@gmail.com'
__status__ = 'Development'


if __name__ == '__main__':
    pass
